@@ -69,6 +69,9 @@ GRegex * 	arv_gv_device_get_url_regex 			(void);
 GSocketAddress *arv_gv_device_get_interface_address  		(ArvGvDevice *device);
 GSocketAddress *arv_gv_device_get_device_address  		(ArvGvDevice *device);
 
+guint		arv_gv_device_get_packet_size (ArvGvDevice *gv_device);
+void		arv_gv_device_set_packet_size (ArvGvDevice *gv_device, guint packet_size);
+
 G_END_DECLS
 
 #endif
