@@ -627,14 +627,24 @@ ArvGstCapsInfos arv_gst_caps_infos[] = {
 	       "video/x-raw-bayer",	8,	8,	0
 	},
 	{
-	       ARV_PIXEL_FORMAT_BAYER_BG_12,
-	       "video/x-raw-bayer, bpp=(int)16, depth=(int)12",
-	       "video/x-raw-bayer",	16,	12,	0
+		ARV_PIXEL_FORMAT_BAYER_GR_8,
+		"video/x-raw-bayer, format=grbg, bpp=(int)8, depth=(int)8",
+		"video/x-raw-bayer",     8,      8,      0
 	},
 	{
-	       ARV_PIXEL_FORMAT_BAYER_BG_12_PACKED,
-	       "video/x-raw-bayer, bpp=(int)12, depth=(int)12",
-	       "video/x-raw-bayer",	12,	12,	0
+		ARV_PIXEL_FORMAT_BAYER_BG_12,
+		"video/x-raw-bayer, bpp=(int)16, depth=(int)12",
+		"video/x-raw-bayer",	16,	12,	0
+	},
+	{
+		ARV_PIXEL_FORMAT_BAYER_GR_12,
+		"video/x-raw-bayer, format=grbg, bpp=(int)16, depth=(int)12",
+		"video/x-raw-bayer",     16,     12,     0
+	},
+	{
+		ARV_PIXEL_FORMAT_BAYER_BG_12_PACKED,
+		"video/x-raw-bayer, bpp=(int)12, depth=(int)12",
+		"video/x-raw-bayer",	12,	12,	0
 	},
 	{
 		ARV_PIXEL_FORMAT_YUV_422_PACKED,
@@ -642,9 +652,9 @@ ArvGstCapsInfos arv_gst_caps_infos[] = {
 		"video/x-raw-yuv",	0,	0,	ARV_MAKE_FOURCC ('U','Y','V','Y')
 	},
 	{
-	       ARV_PIXEL_FORMAT_YUV_422_YUYV_PACKED,
-	       "video/x-raw-yuv, format=(fourcc)YUY2",
-	       "video/x-raw-yuv",	0,	0,	ARV_MAKE_FOURCC ('Y','U','Y','2')
+		ARV_PIXEL_FORMAT_YUV_422_YUYV_PACKED,
+		"video/x-raw-yuv, format=(fourcc)YUY2",
+		"video/x-raw-yuv",	0,	0,	ARV_MAKE_FOURCC ('Y','U','Y','2')
 	},
 	{
 		ARV_PIXEL_FORMAT_RGB_8_PACKED,
