@@ -29,6 +29,10 @@
 #include <arvenumtypes.h>
 #include <stdlib.h>
 #include <math.h>
+#include <glib/gi18n-lib.h>
+#ifdef ARAVIS_WITH_NOTIFY
+#include <libnotify/notify.h>
+#endif
 
 static char *arv_viewer_option_debug_domains = NULL;
 static gboolean arv_viewer_option_auto_socket_buffer = FALSE;
@@ -42,6 +46,8 @@ typedef struct {
 	ArvStream *stream;
 	ArvBuffer *last_buffer;
 
+	const char *pixel_format_string;
+
 	GstElement *pipeline;
 	GstElement *appsrc;
 
@@ -72,6 +78,12 @@ typedef struct {
 
 	guint gain_update_event;
 	guint exposure_update_event;
+
+	gulong video_window_xid;
+
+#ifdef ARAVIS_WITH_NOTIFY
+	NotifyNotification *notification;
+#endif
 } ArvViewer;
 
 double
@@ -268,7 +280,7 @@ gboolean
 arv_viewer_update_gain_cb (void *data)
 {
 	ArvViewer *viewer = data;
-	gint64 gain;
+	double gain;
 
 	gain = arv_camera_get_gain (viewer->camera);
 
@@ -342,25 +354,6 @@ arv_viewer_release_camera (ArvViewer *viewer)
 	viewer->last_timestamp = 0;
 }
 
-static const char *
-arv_enum_to_string (GType type,
-		    guint enum_value)
-{
-	GEnumClass *enum_class;
-	GEnumValue *value;
-	const char *retval = NULL;
-
-	enum_class = g_type_class_ref (type);
-
-	value = g_enum_get_value (enum_class, enum_value);
-	if (value)
-		retval = value->value_nick;
-
-	g_type_class_unref (enum_class);
-
-	return retval;
-}
-
 void
 arv_viewer_snapshot_cb (GtkButton *button, ArvViewer *viewer)
 {
@@ -387,12 +380,22 @@ arv_viewer_snapshot_cb (GtkButton *button, ArvViewer *viewer)
 				    arv_camera_get_device_id (viewer->camera),
 				    viewer->last_buffer->width,
 				    viewer->last_buffer->height,
-				    arv_enum_to_string (ARV_TYPE_PIXEL_FORMAT,
-							viewer->last_buffer->pixel_format),
+				    viewer->pixel_format_string != NULL ? viewer->pixel_format_string : "Unknown",
 				    date_string);
 	path = g_build_filename (g_get_user_special_dir (G_USER_DIRECTORY_PICTURES),
 				 "Aravis", filename, NULL);
 	g_file_set_contents (path, viewer->last_buffer->data, viewer->last_buffer->size, NULL);
+
+#ifdef ARAVIS_WITH_NOTIFY
+	if (viewer->notification) {
+		notify_notification_update (viewer->notification,
+					    "Snapshot saved to Image folder",
+					    path,
+					    "gtk-save");
+		notify_notification_show (viewer->notification, NULL);
+	}
+#endif
+
 	g_free (path);
 	g_free (filename);
 	g_free (date_string);
@@ -404,7 +407,6 @@ arv_viewer_select_camera_cb (GtkComboBox *combo_box, ArvViewer *viewer)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *list_store;
-	GdkWindow *window;
 	GstCaps *caps;
 	GstElement *ffmpegcolorspace;
 	GstElement *ximagesink;
@@ -415,11 +417,14 @@ arv_viewer_select_camera_cb (GtkComboBox *combo_box, ArvViewer *viewer)
 	int width;
 	int height;
 	unsigned int i;
-	gulong window_xid;
 	double frame_rate;
-	gint gain_min, gain_max;
+	double gain_min, gain_max;
 	gboolean auto_gain, auto_exposure;
 	const char *caps_string;
+	gboolean is_exposure_available;
+	gboolean is_exposure_auto_available;
+	gboolean is_gain_available;
+	gboolean is_gain_auto_available;
 
 	g_return_if_fail (viewer != NULL);
 
@@ -460,12 +465,18 @@ arv_viewer_select_camera_cb (GtkComboBox *combo_box, ArvViewer *viewer)
 
 	arv_camera_get_region (viewer->camera, NULL, NULL, &width, &height);
 	pixel_format = arv_camera_get_pixel_format (viewer->camera);
+	viewer->pixel_format_string = arv_camera_get_pixel_format_as_string (viewer->camera);
 	arv_camera_get_exposure_time_bounds (viewer->camera, &viewer->exposure_min, &viewer->exposure_max);
 	arv_camera_get_gain_bounds (viewer->camera, &gain_min, &gain_max);
 	frame_rate = arv_camera_get_frame_rate (viewer->camera);
 	auto_gain = arv_camera_get_gain_auto (viewer->camera) != ARV_AUTO_OFF;
 	auto_exposure = arv_camera_get_gain_auto (viewer->camera) != ARV_AUTO_OFF;
 
+	is_exposure_available = arv_camera_is_exposure_time_available (viewer->camera);
+	is_exposure_auto_available = arv_camera_is_exposure_auto_available (viewer->camera);
+	is_gain_available = arv_camera_is_gain_available (viewer->camera);
+	is_gain_auto_available = arv_camera_is_gain_auto_available (viewer->camera);
+
 	g_signal_handler_block (viewer->gain_hscale, viewer->gain_hscale_changed);
 	g_signal_handler_block (viewer->gain_spin_button, viewer->gain_spin_changed);
 	g_signal_handler_block (viewer->exposure_hscale, viewer->exposure_hscale_changed);
@@ -484,6 +495,12 @@ arv_viewer_select_camera_cb (GtkComboBox *combo_box, ArvViewer *viewer)
 	gtk_entry_set_text (GTK_ENTRY (viewer->frame_rate_entry), string);
 	g_free (string);
 
+	gtk_widget_set_sensitive (viewer->gain_hscale, is_gain_available);
+	gtk_widget_set_sensitive (viewer->gain_spin_button, is_gain_available);
+
+	gtk_widget_set_sensitive (viewer->exposure_hscale, is_exposure_available);
+	gtk_widget_set_sensitive (viewer->exposure_spin_button, is_exposure_available);
+
 	g_signal_handler_unblock (viewer->gain_hscale, viewer->gain_hscale_changed);
 	g_signal_handler_unblock (viewer->gain_spin_button, viewer->gain_spin_changed);
 	g_signal_handler_unblock (viewer->exposure_hscale, viewer->exposure_hscale_changed);
@@ -497,8 +514,13 @@ arv_viewer_select_camera_cb (GtkComboBox *combo_box, ArvViewer *viewer)
 
 	g_signal_handler_block (viewer->auto_gain_toggle, viewer->auto_gain_clicked);
 	g_signal_handler_block (viewer->auto_exposure_toggle, viewer->auto_exposure_clicked);
+	
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (viewer->auto_gain_toggle), auto_gain);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (viewer->auto_exposure_toggle), auto_exposure);
+
+	gtk_widget_set_sensitive (viewer->auto_gain_toggle, is_gain_auto_available);
+	gtk_widget_set_sensitive (viewer->auto_exposure_toggle, is_exposure_auto_available);
+
 	g_signal_handler_unblock (viewer->auto_gain_toggle, viewer->auto_gain_clicked);
 	g_signal_handler_unblock (viewer->auto_exposure_toggle, viewer->auto_exposure_clicked);
 
@@ -541,9 +563,8 @@ arv_viewer_select_camera_cb (GtkComboBox *combo_box, ArvViewer *viewer)
 
 	gst_element_set_state (viewer->pipeline, GST_STATE_PLAYING);
 
-	window = gtk_widget_get_window (viewer->drawing_area);
-	window_xid = GDK_WINDOW_XID (window);
-	gst_x_overlay_set_xwindow_id (GST_X_OVERLAY (ximagesink), window_xid);
+	g_assert (viewer->video_window_xid != 0);
+	gst_x_overlay_set_window_handle (GST_X_OVERLAY (ximagesink), viewer->video_window_xid);
 
 	g_signal_connect (viewer->stream, "new-buffer", G_CALLBACK (arv_viewer_new_buffer_cb), viewer);
 }
@@ -553,6 +574,11 @@ arv_viewer_free (ArvViewer *viewer)
 {
 	g_return_if_fail (viewer != NULL);
 
+#ifdef ARAVIS_WITH_NOTIFY
+	if (viewer->notification)
+		g_object_unref (viewer->notification);
+#endif
+
 	if (viewer->exposure_update_event > 0)
 		g_source_remove (viewer->exposure_update_event);
 	if (viewer->gain_update_event > 0)
@@ -569,6 +595,12 @@ arv_viewer_quit_cb (GtkWidget *widget, ArvViewer *viewer)
 	gtk_main_quit ();
 }
 
+static void
+drawing_area_realize_cb (GtkWidget * widget, ArvViewer *viewer)
+{
+	viewer->video_window_xid = GDK_WINDOW_XID (gtk_widget_get_window (widget));
+}
+
 ArvViewer *
 arv_viewer_new (void)
 {
@@ -581,8 +613,17 @@ arv_viewer_new (void)
 
 	builder = gtk_builder_new ();
 
-	ui_filename = g_build_filename (ARAVIS_DATA_DIR, "arv-viewer.ui", NULL);
-	gtk_builder_add_from_file (builder, ui_filename, NULL);
+	ui_filename = g_build_filename (ARAVIS_DATA_DIR,
+#if GTK_CHECK_VERSION(3,0,0)
+					"arv-viewer-3.ui",
+#else
+					"arv-viewer-2.ui",
+#endif
+					NULL);
+
+	if (!gtk_builder_add_from_file (builder, ui_filename, NULL))
+		g_error ("The user interface file is missing ('%s')", ui_filename);
+
 	g_free (ui_filename);
 
 	viewer->camera_combo_box = GTK_WIDGET (gtk_builder_get_object (builder, "camera_combobox"));
@@ -606,6 +647,9 @@ arv_viewer_new (void)
 
 	gtk_widget_set_no_show_all (viewer->trigger_combo_box, TRUE);
 
+	g_signal_connect (viewer->drawing_area, "realize", G_CALLBACK (drawing_area_realize_cb), viewer);
+	gtk_widget_set_double_buffered (viewer->drawing_area, FALSE);
+
 	gtk_widget_show_all (viewer->main_window);
 
 	g_signal_connect (viewer->main_window, "destroy", G_CALLBACK (arv_viewer_quit_cb), viewer);
@@ -627,6 +671,10 @@ arv_viewer_new (void)
 	viewer->auto_gain_clicked = g_signal_connect (viewer->auto_gain_toggle, "clicked",
 						      G_CALLBACK (arv_viewer_auto_gain_cb), viewer);
 
+#ifdef ARAVIS_WITH_NOTIFY
+	viewer->notification = notify_notification_new (NULL, NULL, NULL);
+#endif
+
 	return viewer;
 }
 
@@ -662,6 +710,10 @@ main (int argc,char *argv[])
 	GOptionContext *context;
 	GError *error = NULL;
 
+	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
+
 	g_thread_init (NULL);
 
 	context = g_option_context_new (NULL);
@@ -682,6 +734,10 @@ main (int argc,char *argv[])
 
 	arv_debug_enable (arv_viewer_option_debug_domains);
 
+#ifdef ARAVIS_WITH_NOTIFY
+	notify_init ("Aravis Viewer");
+#endif
+
 	viewer = arv_viewer_new ();
 
 	arv_viewer_update_device_list_cb (viewer);
@@ -689,6 +745,10 @@ main (int argc,char *argv[])
 
 	gtk_main ();
 
+#ifdef ARAVIS_WITH_NOTIFY
+	notify_uninit ();
+#endif
+
 	/* For debug purpose only */
 	arv_shutdown ();
 
